/*******************************************************************
 * Data Visualization - Foreign News At the Bpi   *
 * Tested in Processing 3                                           *     
 *                                                                  *
 * Author: Yichen Li                                                *
 * Supervisor: George Legrady                                       *
 *                                                                  *
 * Reference: Mouse hover based on Rodger Luo's Basic3D code and    *
 *            accompanying classes and functions. Revision          *
 *            by Rodger Luo.                                        *
 * Purpose: Demonstrate the browsing activity of foreign news at    *
             the Bpi.                                               *
 *******************************************************************/
/* ------- importing stuff -------*/
import peasy.*;
import controlP5.*;
PShader blurr;

// variables for table
Table data;
int numRow;

// variables for the newarticles class
String date, session_id, timestamp, domain, url, title, country, host_ip, host_country;
int country_utc;
float host_lon, host_lat;
ArrayList<Data> newsvisits = new ArrayList<Data>();

//variables for drawing
float distancecirclex, lonoffset, y_axis;
int howfarishomemax, utchomemax, alpha, blur;
PeasyCam cam;
PGraphics pg;
String showText;
color[] list_color = {color(12,2,39),color(16,1,40),color(38,2,46), color(86,0,67),color(117,0,61),color(129,0,56),color(117,5,45), color(246,30,37),color(255,48,32),color(242,85,21),color(252,148,27),color(252,165,32),color(0,0,0)};
color line_color = (0);
String ahead_behind;

void setup() {
  //fullScreen(P3D, SPAN);
  size(1080, 720, P3D);
  cam = new PeasyCam(this, 1080/2,720/2,100,800);
  //pg = createGraphics(40, 40, P3D);
  blurr = loadShader("blur.glsl");
  blurr.set("sigma", map(mouseX, 0, width, 0, 10.0));
  blurr.set("blurSize", (int) map(mouseY, 0, height, 0, 30.0));
  blurr.set("texOffset", 1.0, 1.0);
  //cp5 = new ControlP5(this);
  data = loadTable("w_distance_and_utc_scraped-and-cleaned-logs-bpi-news-2017-08-14-2017-08-21.csv", "header");
  //loads table data into Data class, newsvisits array
  for (TableRow row : data.rows()) {
    String date=row.getString("date");
    String session_id=row.getString("session_id");
    String timestamp=row.getString("timestamp");
    String domain=row.getString("domain");
    String url=row.getString("url");
    String title=row.getString("title");
    String country=row.getString("country");
    String host_ip=row.getString("host_ip");
    String host_country=row.getString("host_country");
    int country_utc=row.getInt("country_utc");
    float country_lat=row.getFloat("country_lat");
    float country_lon=row.getFloat("country_lon");
    float host_lon=row.getFloat("host_lon");
    float host_lat=row.getFloat("host_lat");
    float distance=row.getFloat("distance_from_bpi");
    newsvisits.add(new Data(date,session_id,timestamp,domain,url,title,country,host_ip,host_country,country_utc,country_lat, country_lon, host_lon,host_lat, distance));
  }
  print("input table loaded");
}

void draw(){
  background(245);
  
  //draws "how far is home?" axis and utc offset axis
  howfarishomemax = 1400;
  noFill();
  pushMatrix();
  stroke(line_color);
  strokeWeight(5);
  line(0,0,0,howfarishomemax*3/2-200,0,0);
  strokeWeight(1);
  line(-howfarishomemax*3/2,0,0,howfarishomemax*3/2,0,0);
  rotateZ(radians(90));
  stroke(line_color);
  strokeWeight(1);
  arc(0,0,howfarishomemax/5,howfarishomemax/5,radians(-220),radians(70));
  arc(0,0,howfarishomemax/4,howfarishomemax/4,radians(-220),radians(70));
  arc(0,0,howfarishomemax/3,howfarishomemax/3,radians(-220),radians(70));
  arc(0,0,howfarishomemax/2,howfarishomemax/2,radians(-220),radians(70));
  stroke(110);
  arc(0,0,howfarishomemax,howfarishomemax,radians(-220),radians(70));
  arc(0,0,howfarishomemax*2/3,howfarishomemax*2/3,radians(-220),radians(70));
  stroke(130);
  arc(0,0,howfarishomemax*3/2,howfarishomemax*3/2,radians(-220),radians(70));
  stroke(150);
  arc(0,0,howfarishomemax*2,howfarishomemax*2,radians(-220),radians(70));
  stroke(170);
  arc(0,0,howfarishomemax*5/2,howfarishomemax*5/2,radians(-220),radians(70));
  popMatrix();
  textSize(50);
  fill(0);
  text("HOW FAR IS HOME?",howfarishomemax*3/2-200,0,0);
  pushMatrix();
  rotateX(radians(270));
  text("IS IT YESTERDAY THERE?",howfarishomemax*3/2-200,0,0);
  for (int i = -12; i < 12; i = i+1){
    textSize(15);
    noFill();
    text("utc" + str(i),100*(i-1)+10,10,howfarishomemax);
    stroke(list_color[abs(12-abs(i-1))]);
    line(100*(i-1)+10,0,-howfarishomemax*1/2,100*(i-1)+10,0,howfarishomemax);
  }
  popMatrix();
  //draws "host of website (latitude)"
  utchomemax = howfarishomemax*3/2+200;
  strokeWeight(5);
  line(-howfarishomemax*3/2,0,utchomemax/3,-howfarishomemax*3/2,0,-utchomemax/10);
  strokeWeight(1);
  beginShape();
  vertex(-howfarishomemax*3/2,0,utchomemax/3);
  vertex(-howfarishomemax*3/2,-30,utchomemax/3);
  vertex(-howfarishomemax*3/2,-30,0);
  vertex(-howfarishomemax*3/2,0,0);
  endShape();
  beginShape();
  vertex(-howfarishomemax*3/2,0,-utchomemax/10);
  vertex(-howfarishomemax*3/2,-30,-utchomemax/10);
  vertex(-howfarishomemax*3/2,-30,0);
  vertex(-howfarishomemax*3/2,0,0);
  endShape();
  pushMatrix();
  translate(-howfarishomemax*3/2, 0, utchomemax/3);
  rotateY(radians(270));
  textSize(50);
  textAlign(CENTER);
  text("NH",-utchomemax/6,-35,0);
  text("SH",-utchomemax/3-utchomemax/20,-35,0);
  textAlign(LEFT,BOTTOM);
  text("WHERE IS THE SITE HOSTED?",5,10,0);
  popMatrix();
  //draws "utc of home?" axis
  line(utchomemax,0,0,utchomemax,0,0);
  rotateX(radians(90));
  //rotateZ(radians(180));
  for(int i=0; i<newsvisits.size(); i++){
    distancecirclex = (newsvisits.get(i).distance/5)*cos(map(newsvisits.get(i).country_utc-1,-13,11,0,360));    
    alpha = int(map(newsvisits.get(i).distance,0,10000,50,30));
    textSize(30);
    
    //draws each point
    pushMatrix();
    y_axis = newsvisits.get(i).host_lat*13 + i%20;
    lonoffset = map(newsvisits.get(i).country_lon - 2.35, -102.35, 100-2.35, 10, 100);
    translate((newsvisits.get(i).country_utc-1)*100 + lonoffset, y_axis, distancecirclex);
    //translate(newsvisits.get(i).host_lon,newsvisits.get(i).host_lat,i);
 
// NOTE : attempted to put on a filter, didn't work
    //if (!Float.isNaN(distancecirclex)){
    //  alpha = int(map(distancecirclex,0,360,255,50));
    //  blur = int(map(alpha, 255, 50, 0, 6));
    //}
    //stroke(255,0,0,alpha);
    
    stroke(map(newsvisits.get(i).country.length(), 0, 15, 50, 255), 255-map(newsvisits.get(i).country.length(), 0, 15, 50, 255), map(newsvisits.get(i).country.length(), 0, 15, 50, 255)/2, alpha);
    strokeWeight(2);
    noFill();
    ellipse(0,0,40,40);
    //draws how far from home
    fill(0);
    pushMatrix();
    rotateX(radians(270));
    text(newsvisits.get(i).distance + "km",0,0,0);
    popMatrix();
    
    //draws news title
    pushMatrix();
    rotateZ(radians(270));
    popMatrix();
    
    //draws country name on "how far from home"
    pushMatrix();
    rotateX(radians(270));
    rotateZ(radians(270));
    //rotateY(radians(90));
    //rotateZ(radians(90));
    //filter(BLUR,1);
    text(newsvisits.get(i).country,0,0,0);
    popMatrix();
    
    //mouse action
    PVector mouse_pos = new PVector(mouseX,mouseY);
    PVector screen_pos = new PVector(screenX(0,0,0),screenY(0,0,0));
    float mouseObjectDist = mouse_pos.dist(screen_pos);
    
    if(mouseObjectDist < 5){
      strokeWeight(10);
      point(0,0,0);
      fill(255);
      textSize(30);
      //rotateY(radians(180));
      //rotateZ(radians(180));
      showText = newsvisits.get(i).title;
      cam.beginHUD();
      strokeWeight(5);
      fill(255,0,0);
      textSize(20);
      textAlign(LEFT,BOTTOM);
      if (showText.length() <6){
        showText = "news title not written in Latin alphabet.";
      }
      if (newsvisits.get(i).country_utc -1 < 0){
        ahead_behind = "behind";
      }
      else if (newsvisits.get(i).country_utc -1 > 0){
        ahead_behind = "ahead of ";
      }
      fill(map(newsvisits.get(i).country.length(), 0, 15, 50, 255), 255-map(newsvisits.get(i).country.length(), 0, 15, 50, 255), map(newsvisits.get(i).country.length(), 0, 15, 50, 255)/2);
      text("| " + newsvisits.get(i).domain + " | " + "hosted in " + newsvisits.get(i).host_country+ " | " +showText + "\n| " + newsvisits.get(i).country + " | "+str(abs(round(newsvisits.get(i).distance/870 + 0.5))) + " hours by plane | " + str(abs(newsvisits.get(i).country_utc - 1)) + " hour(s) " + ahead_behind + " 'now'",10,700);
      cam.endHUD();
    }
    popMatrix();
  }
  cam.beginHUD();
  fill(255);
  noStroke();
  rect(0,0,1080,40);
  strokeWeight(5);
  fill(20);
  textSize(20);
  textAlign(LEFT,BOTTOM);
  text("NEWS FROM HOME",20,32);
  textSize(15);
  textAlign(LEFT,TOP);
  textLeading(20);
  text("Bpi browsing activity of foreign news from 08/14 to 08/21, 2017.",20,55);
  text("hover mouse over circle to expand\ntop view: distance of country from BPi\nlongitudinal view: time zone of country\nsection view: website host latitude\nMAT 259A | Yichen Li",20,85);
  cam.endHUD();
}
